@@ -1,26 +1,28 @@
-/* NOVA magnetic tape simulator
+/* nova_mta.c: NOVA magnetic tape simulator
 
-   Copyright (c) 1995, 1996, Robert M Supnik, Digital Equipment Corporation
+   Copyright (c) 1995-1997,
+   Robert M Supnik, Digital Equipment Corporation
    Commercial use prohibited
 
    mta		magnetic tape
 
-   29-Jun-96	RMS	Added unit disable support.
+   18-Jan-97	RMS	New MT format
+   29-Jun-96	RMS	Added unit disable support
 
-   Magnetic tapes are represented as a series of variable 16b records
+   Magnetic tapes are represented as a series of variable records
    of the form:
 
-	byte count low			byte count is little endian
-	byte count high
+	32b byte count			byte count is little endian
 	byte 0
 	byte 1
 	:
 	byte n-2
 	byte n-1
+	32b byte count
 
    If the byte count is odd, the record is padded with an extra byte
-   of junk.  File marks are represented by a byte count of 0; end of
-   tape by end of file.
+   of junk.  File marks are represented by a byte count of 0 and are
+   not duplicated; end of tape by end of file.
 */
 
 #include "nova_defs.h"
@@ -31,8 +33,6 @@
 #define UNIT_WLK	1 << UNIT_V_WLK
 #define USTAT		u3				/* unit status */
 #define UNUM		u4				/* unit number */
-#define SBSIZE		(1 << 16)			/* max space cmd */
-#define SBMASK		(SBSIZE - 1)
 #define DTSIZE		(1 << 14)			/* max data xfer */
 #define DTMASK		(DTSIZE - 1)
 
@@ -107,29 +107,30 @@
 #define STA_MON		(STA_REW | STA_BOT | STA_WLK | STA_RDY | \
 			 STA_PEM)			/* set status chg */
 
-extern unsigned short M[];
+extern unsigned int16 M[];
 extern UNIT cpu_unit;
-extern int int_req, dev_busy, dev_done, dev_disable;
-int mta_ma = 0;						/* memory address */
-int mta_wc = 0;						/* word count */
-int mta_cu = 0;						/* command/unit */
-int mta_sta = 0;					/* status register */
-int mta_ep = 0;						/* enable polling */
-int mta_cwait = 100;					/* command latency */
-int mta_rwait = 100;					/* record latency */
-int mta_svc (UNIT *uptr);
-int mta_reset (DEVICE *dptr);
-int mta_boot (int unitno);
-int mta_attach (UNIT *uptr, char *cptr);
-int mta_detach (UNIT *uptr);
-int mta_updcsta (UNIT *uptr);
-void mta_upddsta (UNIT *uptr, int newsta);
-int mta_vlock (UNIT *uptr, int val);
-extern int sim_activate (UNIT *uptr, int interval);
-extern int sim_cancel (UNIT *uptr);
-extern int sim_is_active (UNIT *uptr);
-extern int attach_unit (UNIT *uptr, char *cptr);
-extern int detach_unit (UNIT *uptr);
+extern int32 int_req, dev_busy, dev_done, dev_disable;
+int32 mta_ma = 0;					/* memory address */
+int32 mta_wc = 0;					/* word count */
+int32 mta_cu = 0;					/* command/unit */
+int32 mta_sta = 0;					/* status register */
+int32 mta_ep = 0;					/* enable polling */
+int32 mta_cwait = 100;					/* command latency */
+int32 mta_rwait = 100;					/* record latency */
+t_stat mta_svc (UNIT *uptr);
+t_stat mta_reset (DEVICE *dptr);
+t_stat mta_boot (int32 unitno);
+t_stat mta_attach (UNIT *uptr, char *cptr);
+t_stat mta_detach (UNIT *uptr);
+int32 mta_updcsta (UNIT *uptr);
+void mta_upddsta (UNIT *uptr, int32 newsta);
+t_stat mta_vlock (UNIT *uptr, int32 val);
+extern t_stat sim_activate (UNIT *uptr, int32 delay);
+extern t_stat sim_cancel (UNIT *uptr);
+extern int32 sim_is_active (UNIT *uptr);
+extern size_t fxread (void *bptr, size_t size, size_t count, FILE *fptr);
+extern size_t fxwrite (void *bptr, size_t size, size_t count, FILE *fptr);
+
 static const int ctype[32] = {				/* c vs r timing */
  0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0,
  0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1  };
@@ -212,10 +213,10 @@ DEVICE mta_dev = {
 
 /* IOT routine */
 
-int mta (int pulse, int code, int AC)
+int32 mta (int32 pulse, int32 code, int32 AC)
 {
 UNIT *uptr;
-int u, c, rval;
+int32 u, c, rval;
 
 rval = 0;
 uptr = mta_dev.units + GET_UNIT(mta_cu);		/* get unit */
@@ -289,13 +290,14 @@ return rval;
    else, do operation, clear busy, set done, interrupt
 */
 
-int mta_svc (UNIT *uptr)
+t_stat mta_svc (UNIT *uptr)
 {
-int c, i, p, rval, u, err;
-int cbc, tbc, wc;
-unsigned short c1, c2, sbuf[SBSIZE];
-unsigned char dbuf[(2 * DTSIZE) + 2];
-const static unsigned char bceof[2] = { 0 };
+int32 c, i, p, u, err;
+t_stat rval;
+t_mtrlnt cbc, tbc, wc;
+unsigned int16 c1, c2;
+unsigned int8 dbuf[2 * DTSIZE];
+static t_mtrlnt bceof = { 0 };
 
 rval = SCPE_OK;
 u = uptr -> UNUM;					/* get unit number */
@@ -333,22 +335,21 @@ case CU_DMODE:						/* drive mode */
 case CU_READ:						/* read */
 case CU_READNS:						/* read non-stop */
 	fseek (uptr -> fileref, uptr -> pos, SEEK_SET);
-	fread (dbuf, sizeof (char), 2, uptr -> fileref); /* read byte count */
-	tbc = ((unsigned int) dbuf[1] << 8) | (unsigned int) dbuf[0];
+	fxread (&tbc, sizeof (t_mtrlnt), 1, uptr -> fileref); /* read byte count */
 	if ((err = ferror (uptr -> fileref)) ||		/* error or eof? */
 	    (feof (uptr -> fileref))) {
 		mta_upddsta (uptr, uptr -> USTAT | STA_RDY | STA_EOT);
 		break;  }
 	if (tbc == 0) {					/* tape mark? */
 		mta_upddsta (uptr, uptr -> USTAT | STA_RDY | STA_EOF);
-		uptr -> pos = uptr -> pos + 2;
+		uptr -> pos = uptr -> pos + sizeof (t_mtrlnt);
 		break;  }
 	cbc = wc * 2;					/* expected bc */
 	if (tbc & 1) mta_sta = mta_sta | STA_ODD;	/* odd byte count? */
 	if (tbc > cbc) mta_sta = mta_sta | STA_WCO;	/* too big? */
 	else {	cbc = tbc;				/* no, use it */
 		wc = (cbc + 1) / 2;  }			/* adjust wc */
-	i = fread (dbuf, sizeof (char), cbc, uptr -> fileref);
+	i = fxread (dbuf, sizeof (int8), cbc, uptr -> fileref);
 	for ( ; i < cbc; i++) dbuf[i] = 0;
 	err = ferror (uptr -> fileref);
 	for (i = p = 0; i < wc; i++) {			/* copy buf to mem */
@@ -357,29 +358,30 @@ case CU_READNS:						/* read non-stop */
 		if (MEM_ADDR_OK (mta_ma)) M[mta_ma] = (c1 << 8) | c2;
 		mta_ma = (mta_ma + 1) & ADDRMASK;  }
 	mta_wc = (mta_wc + wc) & 0177777;
-	uptr -> pos = uptr -> pos + ((tbc + 3) & ~1);
+	uptr -> pos = uptr -> pos + ((tbc + 1) & ~1) +
+		(2 * sizeof (t_mtrlnt));
 	mta_upddsta (uptr, uptr -> USTAT | STA_RDY);
 	break;
 case CU_WRITE:						/* write */
 	fseek (uptr -> fileref, uptr -> pos, SEEK_SET);
 	tbc = wc * 2;					/* io byte count */
-	dbuf[0] = tbc & 0377;
-	dbuf[1] = (tbc >> 8) & 0377;
-	for (i = 0, p = 2; i < wc; i++) {		/* copy to buffer */
+	fxwrite (&tbc, sizeof (t_mtrlnt), 1, uptr -> fileref);
+	for (i = p = 0; i < wc; i++) {		/* copy to buffer */
 		dbuf[p++] = (M[mta_ma] >> 8) & 0377;
 		dbuf[p++] = M[mta_ma] & 0377;
 		mta_ma = (mta_ma + 1) & ADDRMASK;  }
-	fwrite (dbuf, sizeof (char), tbc + 2, uptr -> fileref);
+	fxwrite (dbuf, sizeof (int8), tbc, uptr -> fileref);
+	fxwrite (&tbc, sizeof (t_mtrlnt), 1, uptr -> fileref);
 	err = ferror (uptr -> fileref);
 	mta_wc = 0;
-	uptr -> pos = uptr -> pos + tbc + 2;
+	uptr -> pos = uptr -> pos + tbc + (2 * sizeof (t_mtrlnt));
 	mta_upddsta (uptr, uptr -> USTAT | STA_RDY);
 	break;
 case CU_WREOF:						/* write eof */
 	fseek (uptr -> fileref, uptr -> pos, SEEK_SET);
-	fwrite (bceof, sizeof (char), 2, uptr -> fileref);
+	fxwrite (&bceof, sizeof (t_mtrlnt), 1, uptr -> fileref);
 	err = ferror (uptr -> fileref);
-	uptr -> pos = uptr -> pos + 2;
+	uptr -> pos = uptr -> pos + sizeof (t_mtrlnt);
 	mta_upddsta (uptr, uptr -> USTAT | STA_EOF | STA_RDY);
 	break;
 case CU_ERASE:						/* erase */
@@ -391,44 +393,42 @@ case CU_ERASE:						/* erase */
 case CU_SPACEF:						/* space forward */
 	do {	mta_wc = (mta_wc + 1) & 0177777;	/* incr wc */
 		fseek (uptr -> fileref, uptr -> pos, SEEK_SET);
-		fread (dbuf, sizeof (char), 2, uptr -> fileref); /* read bc */
-		tbc = ((unsigned int) dbuf[1] << 8) | (unsigned int) dbuf[0];
+		fxread (&tbc, sizeof (t_mtrlnt), 1, uptr -> fileref);
 		if ((err = ferror (uptr -> fileref)) ||	/* error or eof? */
 		    (feof (uptr -> fileref))) {
 			mta_upddsta (uptr, uptr -> USTAT | STA_RDY | STA_EOT);
 			break;  }
 		if (tbc == 0) {				/* zero bc? */
 			mta_upddsta (uptr, uptr -> USTAT | STA_RDY | STA_EOF);
-			uptr -> pos = uptr -> pos + 2;
+			uptr -> pos = uptr -> pos + sizeof (t_mtrlnt);
 			break;  }
-		uptr -> pos = uptr -> pos + ((tbc + 3) & ~1);  }
+		uptr -> pos = uptr -> pos + ((tbc + 1) & ~1) +
+			(2 * sizeof (t_mtrlnt));  }
 	while (mta_wc != 0);
 	mta_upddsta (uptr, uptr -> USTAT | STA_RDY);
 	break;
 case CU_SPACER:						/* space reverse */
-	for (i = 0; i < SBSIZE; i++) sbuf[i] = 0;	/* set table = EOF */
-	for (i = 0, p = 0; p < uptr -> pos; ) {		/* build spc table */
-		fseek (uptr -> fileref, p, SEEK_SET);
-		fread (dbuf, sizeof (char), 2, uptr -> fileref);
-		sbuf[i] = ((unsigned int) dbuf[1] << 8) |
-			   (unsigned int) dbuf[0];
-		if ((err = ferror (uptr -> fileref)) ||
-		    (feof (uptr -> fileref))) {		/* error or eof? */
-			uptr -> pos = p;		/* tape ends */
-			break;  }
-		p = p + (((unsigned int) sbuf[i] + 3) & ~1);
-		i = (i + 1) & SBMASK;  }
 	if (uptr -> pos == 0) {				/* at BOT? */
 		mta_upddsta (uptr, uptr -> USTAT | STA_RDY | STA_BOT);
 		break;  }
 	do {	mta_wc = (mta_wc + 1) & 0177777;	/* incr wc */
-		i = (i - 1) & SBMASK;
-		uptr -> pos = uptr -> pos - (((unsigned int) sbuf[i] + 3) & ~1);
-		if (uptr -> pos == 0)			/* start of tape? */
+		fseek (uptr -> fileref, uptr -> pos - sizeof (t_mtrlnt),
+			SEEK_SET);			/* bs to reclnt */
+		fxread (&tbc, sizeof (t_mtrlnt), 1, uptr -> fileref);
+		if ((err = ferror (uptr -> fileref)) ||	/* error or eof? */
+		    (feof (uptr -> fileref))) {
 			mta_upddsta (uptr, uptr -> USTAT | STA_RDY | STA_BOT);
-		if (sbuf[i] == 0)			/* start of prv file? */
+			uptr -> pos = 0;
+			break;  }
+		if (tbc == 0) {				/* zero bc? */
 			mta_upddsta (uptr, uptr -> USTAT | STA_RDY | STA_EOF);
-		if (uptr -> USTAT & (STA_BOT | STA_EOF)) break;  }
+			uptr -> pos = uptr -> pos - sizeof (t_mtrlnt);
+			break;  }
+		uptr -> pos = uptr -> pos - ((tbc + 1) & ~1) -
+			(2 * sizeof (t_mtrlnt));
+		if (uptr -> pos == 0) {			/* start of tape? */
+			mta_upddsta (uptr, uptr -> USTAT | STA_RDY | STA_BOT);
+			break;  }  }
 	while (mta_wc != 0);
 	mta_upddsta (uptr, uptr -> USTAT | STA_RDY);
 	break;
@@ -452,7 +452,7 @@ return rval;
 
 /* Update controller status */
 
-int mta_updcsta (UNIT *uptr)				/* update ctrl */
+int32 mta_updcsta (UNIT *uptr)				/* update ctrl */
 {
 mta_sta = (mta_sta & ~(STA_DYN | STA_CLR | STA_ERR1 | STA_ERR2)) |
 	(uptr -> USTAT & STA_DYN) | STA_SET;
@@ -463,9 +463,9 @@ return mta_sta;
 
 /* Update drive status */
 
-void mta_upddsta (UNIT *uptr, int newsta)		/* drive status */
+void mta_upddsta (UNIT *uptr, int32 newsta)		/* drive status */
 {
-int change;
+int32 change;
 
 if ((uptr -> flags & UNIT_ATT) == 0) newsta = 0;	/* offline? */
 change = (uptr -> USTAT ^ newsta) & STA_MON;		/* changes? */
@@ -481,9 +481,9 @@ return;
 
 /* Reset routine */
 
-int mta_reset (DEVICE *dptr)
+t_stat mta_reset (DEVICE *dptr)
 {
-int u;
+int32 u;
 UNIT *uptr;
 
 dev_busy = dev_busy & ~INT_MTA;				/* clear busy */
@@ -506,9 +506,10 @@ return SCPE_OK;
 
 /* Attach routine */
 
-int mta_attach (UNIT *uptr, char *cptr)
+t_stat mta_attach (UNIT *uptr, char *cptr)
 {
-int r;
+t_stat r;
+
 r = attach_unit (uptr, cptr);
 if (r != SCPE_OK) return r;
 if (!sim_is_active (uptr)) mta_upddsta (uptr, STA_RDY | STA_BOT | STA_PEM |
@@ -518,7 +519,7 @@ return r;
 
 /* Detach routine */
 
-int mta_detach (UNIT* uptr)
+t_stat mta_detach (UNIT* uptr)
 {
 if (!sim_is_active (uptr)) mta_upddsta (uptr, 0);
 return detach_unit (uptr);
@@ -526,7 +527,7 @@ return detach_unit (uptr);
 
 /* Write lock/unlock validate routine */
 
-int mta_vlock (UNIT *uptr, int val)
+t_stat mta_vlock (UNIT *uptr, int32 val)
 {
 if ((uptr -> flags & UNIT_ATT) && val)
 	mta_upddsta (uptr, uptr -> USTAT | STA_WLK);
@@ -540,7 +541,7 @@ return SCPE_OK;
 #define BOOT_UNIT 02020
 #define BOOT_LEN (sizeof (boot_rom) / sizeof (int))
 
-static const int boot_rom[] = {
+static const int32 boot_rom[] = {
 	060222,			/* NIOC 0,MTA		; clear disk */
 	020417,			/* LDA 0,UNIT		; unit */
 	024417,			/* LDA 1,REWIND		; cmd */
@@ -561,10 +562,10 @@ static const int boot_rom[] = {
 	000010			/* REWIND: 10 */
 };
 
-int mta_boot (int unitno)
+t_stat mta_boot (int32 unitno)
 {
-int i;
-extern int saved_PC;
+int32 i;
+extern int32 saved_PC;
 
 for (i = 0; i < BOOT_LEN; i++) M[BOOT_START + i] = boot_rom[i];
 M[BOOT_UNIT] = (unitno & CU_M_UNIT) << CU_V_UNIT;
