@@ -1,9 +1,12 @@
 /* pdp11_cpu.c: PDP-11 CPU simulator
 
-   Copyright (c) 1993, 1994, 1996,
+   Copyright (c) 1993-1998,
    Robert M Supnik, Digital Equipment Corporation
    Commercial use prohibited
 
+   18-Aug-98	RMS	Added CIS support
+   09-May-98	RMS	Fixed bug in DIV overflow test
+   19-Jan-97	RMS	Added RP/RM support
    06-Apr-96	RMS	Added dynamic memory sizing
    29-Feb-96	RMS	Added TM11 support
    17-Jul-94	RMS	Corrected updating of MMR1 if MMR0 locked
@@ -156,109 +159,115 @@
 #define last_pa		(cpu_unit.u4)			/* and RESTOREd */
 #define UNIT_V_18B	(UNIT_V_UF)			/* force 18b addr */
 #define UNIT_18B	(1u << UNIT_V_18B)
-#define UNIT_V_MSIZE	(UNIT_V_UF + 1)			/* dummy */
+#define UNIT_V_CIS	(UNIT_V_UF + 1)		/* CIS present */
+#define UNIT_CIS	(1u << UNIT_V_CIS)
+#define UNIT_V_MSIZE	(UNIT_V_UF + 2)			/* dummy */
 #define UNIT_MSIZE	(1u << UNIT_V_MSIZE)
 
 /* Global state */
 
-unsigned short *M = NULL;				/* address of memory */
-int REGFILE[6][2] = { 0 };				/* R0-R5, two sets */
-int STACKFILE[4] = { 0 };				/* SP, four modes */
-int saved_PC = 0;					/* program counter */
-int R[8] = { 0 };					/* working registers */
-int PSW = 0;						/* PSW */
-  int cm = 0;						/*   current mode */
-  int pm = 0;						/*   previous mode */
-  int rs = 0;						/*   register set */
-  int ipl = 0;						/*   int pri level */
-  int tbit = 0;						/*   trace flag */
-  int N = 0, Z = 0, V = 0, C = 0;			/*   condition codes */
-int wait_state = 0;					/* wait state */
-int trap_req = 0;					/* trap requests */
-int int_req = 0;					/* interrupt requests */
-int PIRQ = 0;						/* programmed int req */
-int SR = 0;						/* switch register */
-int DR = 0;						/* display register */
+unsigned int16 *M = NULL;				/* address of memory */
+int32 REGFILE[6][2] = { 0 };				/* R0-R5, two sets */
+int32 STACKFILE[4] = { 0 };				/* SP, four modes */
+int32 saved_PC = 0;					/* program counter */
+int32 R[8] = { 0 };					/* working registers */
+int32 PSW = 0;						/* PSW */
+  int32 cm = 0;						/*   current mode */
+  int32 pm = 0;						/*   previous mode */
+  int32 rs = 0;						/*   register set */
+  int32 ipl = 0;						/*   int pri level */
+  int32 tbit = 0;						/*   trace flag */
+  int32 N = 0, Z = 0, V = 0, C = 0;			/*   condition codes */
+int32 wait_state = 0;					/* wait state */
+int32 trap_req = 0;					/* trap requests */
+int32 int_req = 0;					/* interrupt requests */
+int32 PIRQ = 0;						/* programmed int req */
+int32 SR = 0;						/* switch register */
+int32 DR = 0;						/* display register */
 fpac_t FR[6] = { 0 };					/* fp accumulators */
-int FPS = 0;						/* fp status */
-int FEC = 0;						/* fp exception code */
-int FEA = 0;						/* fp exception addr */
-int APRFILE[64] = { 0 };				/* PARs/PDRs */
-int MMR0 = 0;						/* MMR0 - status */
-int MMR1 = 0;						/* MMR1 - R+/-R */
-int MMR2 = 0;						/* MMR2 - saved PC */
-int MMR3 = 0;						/* MMR3 - 22b status */
-int isenable = 0, dsenable = 0;				/* i, d space flags */
-int CPUERR = 0;						/* CPU error reg */
-int MEMERR = 0;						/* memory error reg */
-int CCR = 0;						/* cache control reg */
-int HITMISS = 0;					/* hit/miss reg */
-int MAINT = (0 << 9) + (0 << 8) + (4 << 4);		/* maint bit<9> = Q/U */
+int32 FPS = 0;						/* fp status */
+int32 FEC = 0;						/* fp exception code */
+int32 FEA = 0;						/* fp exception addr */
+int32 APRFILE[64] = { 0 };				/* PARs/PDRs */
+int32 MMR0 = 0;						/* MMR0 - status */
+int32 MMR1 = 0;						/* MMR1 - R+/-R */
+int32 MMR2 = 0;						/* MMR2 - saved PC */
+int32 MMR3 = 0;						/* MMR3 - 22b status */
+int32 isenable = 0, dsenable = 0;				/* i, d space flags */
+int32 CPUERR = 0;						/* CPU error reg */
+int32 MEMERR = 0;						/* memory error reg */
+int32 CCR = 0;						/* cache control reg */
+int32 HITMISS = 0;					/* hit/miss reg */
+int32 MAINT = (0 << 9) + (0 << 8) + (4 << 4);		/* maint bit<9> = Q/U */
 							/*  <8> = hwre FP */
 							/*  <6:4> = sys type */
-int stop_trap = 1;					/* stop on trap */
-int stop_vecabort = 1;					/* stop on vec abort */
-int stop_spabort = 1;					/* stop on SP abort */
-int wait_enable = 0;					/* wait state enable */
-int ibkpt_addr = ILL_ADR_FLAG | VAMASK;			/* breakpoint addr */
-int old_PC = 0;						/* previous PC */
+int32 stop_trap = 1;					/* stop on trap */
+int32 stop_vecabort = 1;					/* stop on vec abort */
+int32 stop_spabort = 1;					/* stop on SP abort */
+int32 wait_enable = 0;					/* wait state enable */
+int32 ibkpt_addr = ILL_ADR_FLAG | VAMASK;			/* breakpoint addr */
+int32 old_PC = 0;						/* previous PC */
 jmp_buf save_env;					/* abort handler */
-int dsmask[4] = { MMR3_KDS, MMR3_SDS, 0, MMR3_UDS };	/* dspace enables */
-int int_mask[8] = { INT_IPL0, INT_IPL1, INT_IPL2,	/* interrupt masks */
+int32 dsmask[4] = { MMR3_KDS, MMR3_SDS, 0, MMR3_UDS };	/* dspace enables */
+int32 int_mask[8] = { INT_IPL0, INT_IPL1, INT_IPL2,	/* interrupt masks */
 	INT_IPL3, INT_IPL4, INT_IPL5, INT_IPL6, INT_IPL7 };
-extern int sim_int_char;
+extern int32 sim_int_char;
 
 /* Function declarations */
 
-int cpu_ex (int *vptr, int addr, UNIT *uptr, int sw);
-int cpu_dep (int val, int addr, UNIT *uptr, int sw);
-int cpu_reset (DEVICE *dptr);
-int cpu_svc (UNIT *uptr);
-int cpu_set_size (UNIT *uptr, int value);
-int GeteaB (int spec);
-int GeteaW (int spec);
-int relocR (int addr);
-int relocW (int addr);
-int ReadW (int addr);
-int ReadB (int addr);
-int ReadMW (int addr);
-int ReadMB (int addr);
-void WriteW (int data, int addr);
-void WriteB (int data, int addr);
-void PWriteW (int data, int addr);
-void PWriteB (int data, int addr);
-int iopageR (int *data, int addr, int access);
-int iopageW (int data, int addr, int access);
+t_stat cpu_ex (t_value *vptr, t_addr addr, UNIT *uptr, int32 sw);
+t_stat cpu_dep (t_value val, t_addr addr, UNIT *uptr, int32 sw);
+t_stat cpu_reset (DEVICE *dptr);
+t_stat cpu_svc (UNIT *uptr);
+t_stat cpu_set_size (UNIT *uptr, int32 value);
+int32 GeteaB (int32 spec);
+int32 GeteaW (int32 spec);
+int32 relocR (int32 addr);
+int32 relocW (int32 addr);
+int32 ReadW (int32 addr);
+int32 ReadB (int32 addr);
+int32 ReadMW (int32 addr);
+int32 ReadMB (int32 addr);
+void WriteW (int32 data, int32 addr);
+void WriteB (int32 data, int32 addr);
+void PWriteW (int32 data, int32 addr);
+void PWriteB (int32 data, int32 addr);
+t_stat iopageR (int32 *data, int32 addr, int32 access);
+t_stat iopageW (int32 data, int32 addr, int32 access);
 
-int CPU_rd (int *data, int addr, int access);
-int CPU_wr (int data, int addr, int access);
-int APR_rd (int *data, int addr, int access);
-int APR_wr (int data, int addr, int access);
-int SR_MMR012_rd (int *data, int addr, int access);
-int SR_MMR012_wr (int data, int addr, int access);
-int MMR3_rd (int *data, int addr, int access);
-int MMR3_wr (int data, int addr, int access);
-extern int std_rd (int *data, int addr, int access);
-extern int std_wr (int data, int addr, int access);
-extern int lpt_rd (int *data, int addr, int access);
-extern int lpt_wr (int data, int addr, int access);
-extern int rk_rd (int *data, int addr, int access);
-extern int rk_wr (int data, int addr, int access);
-extern int rk_inta (void);
-extern int rl_rd (int *data, int addr, int access);
-extern int rl_wr (int data, int addr, int access);
-extern int rx_rd (int *data, int addr, int access);
-extern int rx_wr (int data, int addr, int access);
-extern int tm_rd (int *data, int addr, int access);
-extern int tm_wr (int data, int addr, int access);
+t_stat CPU_rd (int32 *data, int32 addr, int32 access);
+t_stat CPU_wr (int32 data, int32 addr, int32 access);
+t_stat APR_rd (int32 *data, int32 addr, int32 access);
+t_stat APR_wr (int32 data, int32 addr, int32 access);
+t_stat SR_MMR012_rd (int32 *data, int32 addr, int32 access);
+t_stat SR_MMR012_wr (int32 data, int32 addr, int32 access);
+t_stat MMR3_rd (int32 *data, int32 addr, int32 access);
+t_stat MMR3_wr (int32 data, int32 addr, int32 access);
+extern t_stat std_rd (int32 *data, int32 addr, int32 access);
+extern t_stat std_wr (int32 data, int32 addr, int32 access);
+extern t_stat lpt_rd (int32 *data, int32 addr, int32 access);
+extern t_stat lpt_wr (int32 data, int32 addr, int32 access);
+extern t_stat rk_rd (int32 *data, int32 addr, int32 access);
+extern t_stat rk_wr (int32 data, int32 addr, int32 access);
+extern int32 rk_inta (void);
+extern t_stat rl_rd (int32 *data, int32 addr, int32 access);
+extern t_stat rl_wr (int32 data, int32 addr, int32 access);
+extern t_stat rp_rd (int32 *data, int32 addr, int32 access);
+extern t_stat rp_wr (int32 data, int32 addr, int32 access);
+extern int32 rp_inta (void);
+extern t_stat rx_rd (int32 *data, int32 addr, int32 access);
+extern t_stat rx_wr (int32 data, int32 addr, int32 access);
+extern t_stat tm_rd (int32 *data, int32 addr, int32 access);
+extern t_stat tm_wr (int32 data, int32 addr, int32 access);
+extern t_stat sim_activate (UNIT *uptr, int32 delay);
 
 /* Auxiliary data structures */
 
 struct iolink {						/* I/O page linkage */
-	int	low;					/* low I/O addr */
-	int	high;					/* high I/O addr */
-	int	(*read)();				/* read routine */
-	int	(*write)();  };				/* write routine */
+	int32	low;					/* low I/O addr */
+	int32	high;					/* high I/O addr */
+	t_stat	(*read)();				/* read routine */
+	t_stat	(*write)();  };				/* write routine */
 
 struct iolink iotable[] = {
 	{ 017777740, 017777777, &CPU_rd, &CPU_wr },
@@ -266,6 +275,7 @@ struct iolink iotable[] = {
 	{ 017777514, 017777517, &lpt_rd, &lpt_wr },
 	{ 017777400, 017777417, &rk_rd, &rk_wr },
 	{ 017774400, 017774411, &rl_rd, &rl_wr },
+	{ 017776700, 017776753, &rp_rd, &rp_wr },
 	{ 017777170, 017777173, &rx_rd, &rx_wr },
 	{ 017772520, 017772533, &tm_rd, &tm_wr },
 	{ 017777600, 017777677, &APR_rd, &APR_wr },
@@ -274,25 +284,25 @@ struct iolink iotable[] = {
 	{ 017772516, 017772517, &MMR3_rd, &MMR3_wr },
 	{ 0, 0, NULL }  };
 
-int int_vec[32] = {					/* int req to vector */
+int32 int_vec[32] = {					/* int req to vector */
 	0, 0, 0, VEC_PIRQ, VEC_CLK, 0, 0, VEC_PIRQ,
-	VEC_RK, VEC_RL, VEC_RX, VEC_TM, 0, 0, 0, VEC_PIRQ,
+	VEC_RK, VEC_RL, VEC_RX, VEC_TM, VEC_RP, 0, 0, VEC_PIRQ,
 	VEC_TTI, VEC_TTO, VEC_PTR, VEC_PTP, VEC_LPT, 0, 0, 0,
 	0, 0, 0, 0, VEC_PIRQ, VEC_PIRQ, VEC_PIRQ, VEC_PIRQ  };
 
-int (*int_ack[32])() = {				/* int ack routines */
+int32 (*int_ack[32])() = {				/* int ack routines */
 	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
-	&rk_inta, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
+	&rk_inta, NULL, NULL, NULL, &rp_inta, NULL, NULL, NULL,
 	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
 	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL  };
 
-int trap_vec[TRAP_V_MAX] = {				/* trap req to vector */
+int32 trap_vec[TRAP_V_MAX] = {				/* trap req to vector */
 	VEC_RED, VEC_ODD, VEC_MME, VEC_NXM,
 	VEC_PAR, VEC_PRV, VEC_ILL, VEC_BPT,
 	VEC_IOT, VEC_EMT, VEC_TRAP, VEC_TRC,
 	VEC_YEL, VEC_PWRFL, VEC_FPE };
 
-int trap_clear[TRAP_V_MAX] = {				/* trap clears */
+int32 trap_clear[TRAP_V_MAX] = {				/* trap clears */
 	TRAP_RED+TRAP_PAR+TRAP_YEL+TRAP_TRC,
 	TRAP_ODD+TRAP_PAR+TRAP_YEL+TRAP_TRC,
 	TRAP_MME+TRAP_PAR+TRAP_YEL+TRAP_TRC,
@@ -471,6 +481,7 @@ REG cpu_reg[] = {
 	{ GRDATA (UDPAR7, APRFILE[077], 8, 16, 16) },
 	{ GRDATA (UDPDR7, APRFILE[077], 8, 16, 0) },
 	{ FLDATA (18B_ADDR, cpu_unit.flags, UNIT_V_18B), REG_HRO },
+	{ FLDATA (CIS, cpu_unit.flags, UNIT_V_CIS), REG_HRO },
 	{ ORDATA (OLDPC, old_PC, 16), REG_RO },
 	{ ORDATA (BREAK, ibkpt_addr, 17) },
 	{ ORDATA (WRU, sim_int_char, 8) },
@@ -479,6 +490,8 @@ REG cpu_reg[] = {
 MTAB cpu_mod[] = {
 	{ UNIT_18B, UNIT_18B, "18b addressing", "18B", NULL },
 	{ UNIT_18B, 0, NULL, "22B", NULL },
+	{ UNIT_CIS, UNIT_CIS, "CIS", "CIS", NULL },
+	{ UNIT_CIS, 0, "No CIS", "NOCIS", NULL },
 	{ UNIT_MSIZE, 16384, NULL, "16K", &cpu_set_size},
 	{ UNIT_MSIZE, 32768, NULL, "32K", &cpu_set_size},
 	{ UNIT_MSIZE, 49152, NULL, "48K", &cpu_set_size},
@@ -506,19 +519,18 @@ DEVICE cpu_dev = {
 	&cpu_ex, &cpu_dep, &cpu_reset,
 	NULL, NULL, NULL };
 
-int sim_instr (void)
+t_stat sim_instr (void)
 {
-extern int sim_interval;
+extern int32 sim_interval;
 extern UNIT *sim_clock_queue;
-register int IR, srcspec, srcreg, dstspec, dstreg;
-register int src, src2, dst;
-register int i, t, sign, oldrs, trapnum;
-int reason, abortval;
-volatile int trapea;
-extern int sim_process_event (void);
-extern int sim_activate (UNIT *uptr, int interval);
-extern int reset_all (int start_device);
-void fp11 (int IR);
+register int32 IR, srcspec, srcreg, dstspec, dstreg;
+register int32 src, src2, dst;
+register int32 i, t, sign, oldrs, trapnum;
+int32 abortval;
+volatile int32 trapea;
+t_stat reason;
+void fp11 (int32 IR);
+void cis11 (int32 IR);
 
 /* Restore register state
 
@@ -1128,7 +1140,7 @@ case 006:						/* ADD */
 	else PWriteW (dst, last_pa);
 	break;
 
-/* Opcode 07: EIS, FIS (not implemented), CIS (not implemented)
+/* Opcode 07: EIS, FIS (not implemented), CIS
 
    Notes:
    - The code assumes that the host int length is at least 32 bits.
@@ -1174,7 +1186,7 @@ case 007:
 		if (GET_SIGN_W (src2)) src2 = src2 | ~077777;
 		if (GET_SIGN_W (R[srcspec])) src = src | ~017777777777;
 		dst = src / src2;
-		if ((dst >= 077777) || (dst < -0100000)) {
+		if ((dst > 077777) || (dst < -0100000)) {
 			V = 1;
 			C = 0;
 			break;  }
@@ -1250,7 +1262,8 @@ case 007:
 		setTRAP (TRAP_ILL);
 		break;
 	case 6:						/* CIS - not impl */
-		setTRAP (TRAP_ILL);
+		if (cpu_unit.flags & UNIT_CIS) cis11 (IR);
+		else setTRAP (TRAP_ILL);
 		break;
 	case 7:						/* SOB */
 		R[srcspec] = (R[srcspec] - 1) & 0177777;
@@ -1624,9 +1637,9 @@ return reason;
 
 /* Effective address calculation for words */
 
-int GeteaW (int spec)
+int32 GeteaW (int32 spec)
 {
-int adr, reg, ds;
+int32 adr, reg, ds;
 
 reg = spec & 07;					/* register number */
 ds = (reg == 7)? isenable: dsenable;			/* dspace if not PC */
@@ -1672,9 +1685,9 @@ case 7:							/* @d(R) */
 
 /* Effective address calculation for bytes */
 
-int GeteaB (int spec)
+int32 GeteaB (int32 spec)
 {
-int adr, reg, ds, delta;
+int32 adr, reg, ds, delta;
 
 reg = spec & 07;					/* reg number */
 ds = (reg == 7)? isenable: dsenable;			/* dspace if not PC */
@@ -1729,9 +1742,9 @@ case 7:							/* @d(R) */
 	data	=	data read from memory or I/O space
 */
 
-int ReadW (int va)
+int32 ReadW (int32 va)
 {
-int pa, data;
+int32 pa, data;
 
 if (va & 1) {						/* odd address? */
 	setCPUERR (CPUE_ODD);
@@ -1747,9 +1760,9 @@ if (iopageR (&data, pa, READ) != SCPE_OK) {		/* invalid I/O addr? */
 return data;
 }
 
-int ReadB (int va)
+int32 ReadB (int32 va)
 {
-int pa, data;
+int32 pa, data;
 
 pa = relocR (va);					/* relocate */
 if (pa < MEMSIZE) return (va & 1? M[pa >> 1] >> 8: M[pa >> 1]) & 0377;
@@ -1762,9 +1775,9 @@ if (iopageR (&data, pa, READ) != SCPE_OK) {		/* invalid I/O addr? */
 return ((va & 1)? data >> 8: data) & 0377;
 }
 
-int ReadMW (int va)
+int32 ReadMW (int32 va)
 {
-int data;
+int32 data;
 
 if (va & 1) {						/* odd address? */
 	setCPUERR (CPUE_ODD);
@@ -1780,9 +1793,9 @@ if (iopageR (&data, last_pa, READ) != SCPE_OK) {	/* invalid I/O addr? */
 return data;
 }
 
-int ReadMB (int va)
+int32 ReadMB (int32 va)
 {
-int data;
+int32 data;
 
 last_pa = relocW (va);					/* reloc, wrt chk */
 if (last_pa < MEMSIZE)
@@ -1805,9 +1818,9 @@ return ((va & 1)? data >> 8: data) & 0377;
    Outputs: none
 */
 
-void WriteW (int data, int va)
+void WriteW (int32 data, int32 va)
 {
-int pa;
+int32 pa;
 
 if (va & 1) {						/* odd address? */
 	setCPUERR (CPUE_ODD);
@@ -1825,9 +1838,9 @@ if (iopageW (data, pa, WRITE) != SCPE_OK) {		/* invalid I/O addr? */
 return;
 }
 
-void WriteB (int data, int va)
+void WriteB (int32 data, int32 va)
 {
-int pa;
+int32 pa;
 
 pa = relocW (va);					/* relocate */
 if (pa < MEMSIZE) {					/* memory address? */
@@ -1843,7 +1856,7 @@ if (iopageW (data, pa, WRITEB) != SCPE_OK) {		/* invalid I/O addr? */
 return;
 }
 
-void PWriteW (int data, int pa)
+void PWriteW (int32 data, int32 pa)
 {
 if (pa < MEMSIZE) {					/* memory address? */
 	M[pa >> 1] = data;
@@ -1857,7 +1870,7 @@ if (iopageW (data, pa, WRITE) != SCPE_OK) {		/* invalid I/O addr? */
 return;
 }
 
-void PWriteB (int data, int pa)
+void PWriteB (int32 data, int32 pa)
 {
 if (pa < MEMSIZE) {					/* memory address? */
 	if (pa & 1) M[pa >> 1] = (M[pa >> 1] & 0377) | (data << 8);
@@ -1886,9 +1899,9 @@ return;
    - Aborts must update MMR0<15:13,6:1> if updating is enabled
 */
 
-int relocR (int va)
+int32 relocR (int32 va)
 {
-int dbn, plf, apridx, apr, pa;
+int32 dbn, plf, apridx, apr, pa;
 
 if (MMR0 & MMR0_MME) {					/* if mmgt */
 	apridx = (va >> VA_V_APF) & 077;		/* index into APR */
@@ -1926,9 +1939,9 @@ return pa;
    - Aborts must update MMR0<15:13,6:1> if updating is enabled
 */
 
-int relocW (int va)
+int32 relocW (int32 va)
 {
-int dbn, plf, apridx, apr, pa;
+int32 dbn, plf, apridx, apr, pa;
 
 if (MMR0 & MMR0_MME) {					/* if mmgt */
 	apridx = (va >> VA_V_APF) & 077;		/* index into APR */
@@ -1967,9 +1980,9 @@ return pa;
    On aborts, this routine returns -1
 */
 
-int relocC (int va, int sw)
+int32 relocC (int32 va, int32 sw)
 {
-int mode, dbn, plf, apridx, apr, pa;
+int32 mode, dbn, plf, apridx, apr, pa;
 
 if (MMR0 & MMR0_MME) {					/* if mmgt */
 	if (sw & SWMASK ('K')) mode = KERNEL;
@@ -2004,9 +2017,9 @@ return pa;
 	status	=	SCPE_OK or SCPE_NXM
 */
 
-int iopageR (int *data, int pa, int access)
+t_stat iopageR (int32 *data, int32 pa, int32 access)
 {
-int stat;
+t_stat stat;
 struct iolink *p;
 
 for (p = &iotable[0]; p -> low != 0; p++ ) {
@@ -2017,9 +2030,9 @@ for (p = &iotable[0]; p -> low != 0; p++ ) {
 return SCPE_NXM;
 }
 
-int iopageW (int data, int pa, int access)
+t_stat iopageW (int32 data, int32 pa, int32 access)
 {
-int stat;
+t_stat stat;
 struct iolink *p;
 
 for (p = &iotable[0]; p -> low != 0; p++ ) {
@@ -2041,7 +2054,7 @@ return SCPE_NXM;
    MMR3 17777516	read/write, certain bits unimplemented
 */
 
-int SR_MMR012_rd (int *data, int pa, int access)
+t_stat SR_MMR012_rd (int32 *data, int32 pa, int32 access)
 {
 switch ((pa >> 1) & 3) {				/* decode pa<2:1> */
 case 0:							/* SR */
@@ -2058,7 +2071,7 @@ case 3:							/* MMR2 */
 	return SCPE_OK;  }				/* end switch pa */
 }
 
-int SR_MMR012_wr (int data, int pa, int access)
+t_stat SR_MMR012_wr (int32 data, int32 pa, int32 access)
 {
 switch ((pa >> 1) & 3) {				/* decode pa<2:1> */
 case 0:							/* DR */
@@ -2073,13 +2086,13 @@ default:						/* MMR1, MMR2 */
 	return SCPE_OK;  }				/* end switch pa */
 }
 
-int MMR3_rd (int *data, int pa, int access)		/* MMR3 */
+t_stat MMR3_rd (int32 *data, int32 pa, int32 access)		/* MMR3 */
 {
 *data = MMR3 & MMR3_IMP;
 return SCPE_OK;
 }
 
-int MMR3_wr (int data, int pa, int access)		/* MMR3 */
+t_stat MMR3_wr (int32 data, int32 pa, int32 access)		/* MMR3 */
 {
 if (pa & 1) return SCPE_OK;
 MMR3 = data & MMR3_RW;
@@ -2107,9 +2120,9 @@ return SCPE_OK;
    Note that the W bit is read only; it is cleared by any write to an APR
 */
 
-int APR_rd (int *data, int pa, int access)
+t_stat APR_rd (int32 *data, int32 pa, int32 access)
 {
-int left, idx;
+t_stat left, idx;
 
 idx = (pa >> 1) & 017;					/* dspace'page */
 left = (pa >> 5) & 1;					/* PDR vs PAR */
@@ -2119,9 +2132,9 @@ if (pa & 0400) idx = idx | 040;				/* 1 for user only */
 return SCPE_OK;
 }
 
-int APR_wr (int data, int pa, int access)
+t_stat APR_wr (int32 data, int32 pa, int32 access)
 {
-int left, idx, curr;
+int32 left, idx, curr;
 
 idx = (pa >> 1) & 017;					/* dspace'page */
 left = (pa >> 5) & 1;					/* PDR vs PAR */
@@ -2148,7 +2161,7 @@ return SCPE_OK;
    PSW		17777776	read/write, with side effects
 */
 
-int CPU_rd (int *data, int pa, int access)
+t_stat CPU_rd (int32 *data, int32 pa, int32 access)
 {
 switch ((pa >> 1) & 017) {				/* decode pa<4:1> */
 case 2: 						/* MEMERR */
@@ -2158,7 +2171,7 @@ case 2: 						/* MEMERR */
 case 3:							/* CCR */
 	*data = CCR;
 	return SCPE_OK;
-case 4:							/* MAINT */
+case 4:							/* MAint */
 	*data = MAINT;
 	return SCPE_OK;
 case 5:							/* Hit/miss */
@@ -2183,9 +2196,9 @@ return SCPE_NXM;					/* unimplemented */
 
 /* CPU control registers, continued */
 
-int CPU_wr (int data, int pa, int access)
+t_stat CPU_wr (int32 data, int32 pa, int32 access)
 {
-int i, pl, curr, oldrs;
+int32 i, pl, curr, oldrs;
 
 switch ((pa >> 1) & 017) {				/* decode pa<4:1> */
 case 2: 						/* MEMERR */
@@ -2260,22 +2273,25 @@ return SCPE_NXM;					/* unimplemented */
 
 /* Reset routine */
 
-int cpu_reset (DEVICE *dptr)
+t_stat cpu_reset (DEVICE *dptr)
 {
 PIRQ = MMR0 = MMR1 = MMR2 = MMR3 = 0;
 DR = CPUERR = MEMERR = CCR = HITMISS = 0;
 PSW = 000340;
 trap_req = 0;
 wait_state = 0;
-if (M == NULL) M = calloc (MEMSIZE >> 1, sizeof (unsigned short));
+if (M == NULL) M = calloc (MEMSIZE >> 1, sizeof (unsigned int16));
 if (M == NULL) return SCPE_MEM;
 return cpu_svc (&cpu_unit);
 }
 
 /* Memory examine */
 
-int cpu_ex (int *vptr, int addr, UNIT *uptr, int sw)
+t_stat cpu_ex (t_value *vptr, t_addr addr, UNIT *uptr, int32 sw)
 {
+int32 iodata;
+t_stat stat;
+
 if (vptr == NULL) return SCPE_ARG;
 if (sw & SWMASK ('V')) {				/* -v */
 	if (addr >= VASIZE) return SCPE_NXM;
@@ -2285,12 +2301,14 @@ if (addr < MEMSIZE) {
 	*vptr = M[addr >> 1] & 0177777;
 	return SCPE_OK;  }
 if (addr < IOPAGEBASE) return SCPE_NXM;
-return iopageR (vptr, addr, READC);
+stat = iopageR (&iodata, addr, READC);
+*vptr = iodata;
+return stat;
 }
 
 /* Memory deposit */
 
-int cpu_dep (int val, int addr, UNIT *uptr, int sw)
+t_stat cpu_dep (t_value val, t_addr addr, UNIT *uptr, int32 sw)
 {
 if (sw & SWMASK ('V')) {				/* -v */
 	if (addr >= VASIZE) return SCPE_NXM;
@@ -2300,12 +2318,12 @@ if (addr < MEMSIZE) {
 	M[addr >> 1] = val & 0177777;
 	return SCPE_OK;  }
 if (addr < IOPAGEBASE) return SCPE_NXM;
-return iopageW (val, addr, WRITEC);
+return iopageW ((int32) val, addr, WRITEC);
 }
 
 /* Breakpoint service */
 
-int cpu_svc (UNIT *uptr)
+t_stat cpu_svc (UNIT *uptr)
 {
 if ((ibkpt_addr & ~ILL_ADR_FLAG) == save_ibkpt) ibkpt_addr = save_ibkpt;
 save_ibkpt = -1;
@@ -2314,18 +2332,17 @@ return SCPE_OK;
 
 /* Memory allocation */
 
-int cpu_set_size (UNIT *uptr, int value)
+t_stat cpu_set_size (UNIT *uptr, int32 value)
 {
-int i, clim, mc = 0;
-unsigned short *nM = NULL;
-extern int get_yn (char *ques, int deflt);
+int32 i, clim, mc = 0;
+unsigned int16 *nM = NULL;
 
 if ((value <= 0) || (value > MAXMEMSIZE) || ((value & 07777) != 0))
 	return SCPE_ARG;
 for (i = value; i < MEMSIZE; i = i + 2)	mc = mc | M[i >> 1];
 if ((mc != 0) && !get_yn ("Really truncate memory [N]?", FALSE))
 	return SCPE_OK;
-nM = calloc (value >> 1, sizeof (unsigned short));
+nM = calloc (value >> 1, sizeof (unsigned int16));
 if (nM == NULL) return SCPE_MEM;
 clim = (value < MEMSIZE)? value: MEMSIZE;
 for (i = 0; i < clim; i = i + 2) nM[i >> 1] = M[i >> 1];
